#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import os
import sys
import re
import pwd
import subprocess

if sys.version_info < (3, ):
    bytes = str
    PY3 = False
else:
    unicode = str
    PY3 = True

#add lib path
playbook_file = os.path.realpath(sys.argv[0])
playbook_path = os.path.split(playbook_file)[0]

lib_path = os.path.join(playbook_path, 'library')
if os.path.exists(lib_path) is True:
    sys.path.insert(0, lib_path)
    lib_path = os.path.join(lib_path, 'securecat')
    if os.path.exists(lib_path) is True:
        sys.path.insert(0, lib_path)
        modulepath = os.environ.get('CSC_MODULE_PATH')
        if modulepath is None:
            os.environ['CSC_MODULE_PATH'] = lib_path

from utils import playbook
from linux.mongo_conf_parser import MongoParser


class MyPlaybook(playbook.Playbook):
    def set_extra_vars_pre(self):
        self._runcontext.set_extra_var('PID', '')

    def get_config_file(self, cmd_list):
        i = 0
        cnf_file = ''
        for p in cmd_list:
            i = i + 1
            if p in ['-f', '--config'] or p.startswith('--config'):
                if "=" in p:
                    cnf_file = p.split("=")[1]
                else:
                    cnf_file = p.strip().split(" ")[-1]
                break
        if i == len(cmd_list):
            cnf_file = ''
        else:
            cnf_file = cmd_list[i] if not cmd_list[i].startswith('-') else cmd_list[i-1].split('=', 1)[1]
            cnf_file = os.path.realpath(cnf_file)
            if not os.path.exists(cnf_file):
                raise Exception('mongod config file %s is not find!' % cnf_file)

        return cnf_file

    def run_cmd(self, cmd):
        p = subprocess.Popen(cmd, env=os.environ, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        self._runcontext.sleep()
        output = p.stdout.read().strip()
        p.stdout.close()

        if PY3:
            output = output.decode('utf-8')

        return output

    def set_extra_vars_post(self):
        """
        处理流程
        1：通过MONGODB_PID，找到对应的配置文件;
        2：通过配置文件获取相应的参数；
        3：更新MONGODB_PORT值
        """
        var_mongodb_pid = self._runcontext.get_extra_var('PID')
        var_mongodb_pid = self._runcontext.replace_inject_characters(str(var_mongodb_pid))

        if var_mongodb_pid == '':
            raise Exception('Must specify extra variable MONGODB_PID.')

        #解析默认配置
        mongodcmd = '/proc/%s/exe' % var_mongodb_pid
        mongodcmd = os.readlink(mongodcmd)

        if not os.path.exists(mongodcmd):
            raise Exception('File %s is not exists.' % mongodcmd)
        if re.search(r';', mongodcmd) or re.search(r'<', mongodcmd) or re.search(r'>', mongodcmd) or re.search(r'\\n', mongodcmd):
            raise Exception('The command contains special characters ;<>\\n, which may lead to privilege escalation')
        self._runcontext.log_debug('mongod = %s' % mongodcmd)
        self._runcontext.set_extra_var('MONGODB_MONGOD', mongodcmd)

        # get mongod owner
        mongod_owner = pwd.getpwuid(os.stat(mongodcmd).st_uid).pw_name
        self._runcontext.set_extra_var('MONGODB_OWNER', mongod_owner)


        mongodcmd = 'ps -wp %s -o cmd | grep -iv cmd' % var_mongodb_pid
        self._runcontext.log_debug('mongod command = %s' % mongodcmd)

        output = self.run_cmd(mongodcmd)
        self._runcontext.log_debug(output)

        # deal with mongdb start up commandline
        cnf_file = ''
        cmd_list = output.split()
        cnf_file = self.get_config_file(cmd_list)
        self._runcontext.log_debug('mongodb cmdline = %s' % output)
        self._runcontext.log_debug('mongodb config file = %s' % cnf_file)
        self._runcontext.set_extra_var('MONGODB_COMMAND', output)
        self._runcontext.set_extra_var('MONGODB_CONFIG', cnf_file)

        parser = MongoParser()
        mongodb_port = ''
        parser.set_conf_file(cnf_file)
        parser.run_parser()
        parser.set_extra_cmdline(cmd_list)
        self._runcontext.log_debug('mongodb config parser = %s' % parser)
        self._runcontext.set_parameter('_mongo_conf_parse', parser)

        if '--port' in cmd_list:
            pos = cmd_list.index('--port')
            mongodb_port = cmd_list[pos + 1]
        else:
            if parser.conf_file != "":
                mongodb_port = str(parser.get_conf_value(['net', 'port'], 'port'))

        if mongodb_port == "":
            mongodb_port = '27017'

        self._runcontext.log_debug('mongodb port = %s' % mongodb_port)
        self._runcontext.set_extra_var('MONGODB_PORT', mongodb_port)

    def get_tasks(self, dir):
        ret = []

        ret.append([dir, 'r_2_1_4_2.py'])
        ret.append([dir, 'r_2_1_4_3.py'])

        ret.append([dir, 'r_2_1_4_5.py'])
        ret.append([dir, 'r_2_1_4_7.py'])
        ret.append([dir, 'r_2_1_4_8.py'])
        ret.append([dir, 'r_2_1_4_13.py'])
        ret.append([dir, 'r_2_1_4_14.py'])
        ret.append([dir, 'r_2_1_4_15.py'])

        ret.append([dir, 'r_2_1_5_1.py'])
        ret.append([dir, 'r_2_1_5_4.py'])
        ret.append([dir, 'r_2_1_5_5.py'])
        ret.append([dir, 'r_2_1_5_6.py'])
        ret.append([dir, 'r_2_1_5_7.py'])
        ret.append([dir, 'r_2_1_5_8.py'])

        ret.append([dir, 'r_2_1_6_1.py'])
        ret.append([dir, 'r_2_1_6_2.py'])

        ret.append([dir, 'r_2_2_1_1.py'])
        ret.append([dir, 'r_2_2_1_2.py'])
        ret.append([dir, 'r_2_2_1_3.py'])
        ret.append([dir, 'r_2_2_1_4.py'])
        ret.append([dir, 'r_2_2_1_5.py'])

        ret.append([dir, 'r_3_1_1.py'])
        ret.append([dir, 'r_3_1_2.py'])
        ret.append([dir, 'r_3_1_3.py'])

        return ret


if __name__ == '__main__':

    myplaybook = MyPlaybook()
    myplaybook.set_role('gp')
    myplaybook.run()
