# -*- coding: UTF-8 -*-
import re


def get_tags():
    return ["2", "2.1", "2.1.4", "2.1.4.5"]


def ip_verification(ip):
    pattern = re.compile(r'^(?=(\b|\D))(((\d{1,2})|(1\d{1,2})|(2[0-4]\d)|(25[0-5]))\.){3}((\d{1,2})|(1\d{1,2})|(2[0-4]\d)|(25[0-5]))(?=(\b|\D))$')
    ip_list = ip.split(',')
    ip_list = [x.strip() for x in ip_list]
    result_list = []
    for single_ip in ip_list:
        if pattern.match(single_ip) is not None and single_ip not in ['0.0.0.0', '*']:
            result_list.append(True)
        else:
            result_list.append(False)
    if False not in result_list:
        return True
    return False


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["systemvalue1"] = ""
    ret_code = -1

    var_mongo_parser = runcontext.get_parameter('_mongo_conf_parse')
    ret_cmd_param, ret_cmd_value = var_mongo_parser.get_cmd_value(["--bind_ip"])
    runcontext.log_debug("ret_cmd_param: %s  ret_cmd_value: %s" % (ret_cmd_param, ret_cmd_value))
    if len(ret_cmd_value) != 0:
        tmp_vars["systemvalue1"] = ret_cmd_value
        ip_valid = ip_verification(ret_cmd_value)
        if ip_valid:
            tmp_vars["checkresult1"] = "pass"
    else:
        yml_key_list = ["net", "bindIp"]
        key_list = ["bind_ip"]
        ret_conf_value = var_mongo_parser.get_conf_value(yml_key_list, key_list)
        runcontext.log_debug("ret_conf_value: %s" % str(ret_conf_value))
        if ret_conf_value is not None:
            tmp_vars["systemvalue1"] = ret_conf_value
            ip_valid = ip_verification(ret_conf_value)
            if ip_valid:
                tmp_vars["checkresult1"] = "pass"

    if tmp_vars["checkresult1"] == "pass":
        ret_code = 0
        tmp_vars["policyresult"] = "pass"

    rc, out = curtemplate.render(runcontext.get_template_file('r_2_1_4_5.j2'), **tmp_vars)

    if rc == False:
        return (-2, "Failed to render template.%s" % out)

    return ret_code, out
