# -*- coding: UTF-8 -*-
import re
import os
import sys
import subprocess

if sys.version_info < (3, ):
    bytes = str
    PY3 = False
else:
    unicode = str
    PY3 = True


def run_cmd(cmd):
    p = subprocess.Popen(cmd, env=os.environ, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
    output = p.stdout.read().strip()
    p.stdout.close()
    if PY3:
        output = output.decode('utf-8')
    return output


def dbversion_compare(db_version_list):
    """db version > 3.0 return True"""
    ret_flag = False
    for i in range(len(db_version_list)):
        if int(db_version_list[i]) > 3:
            ret_flag = True
            break
        elif int(db_version_list[0]) == 3:
            if len(db_version_list[1:]) <= 0:
                ret_flag = False
                break
            else:
                if int(db_version_list[1]) > 0:
                    ret_flag = True
                    break
                else:
                    ret_flag = False
                    break
        else:
            ret_flag = False
            break
    return ret_flag


def get_tags():
    return ["2", "2.1", "2.1.4", "2.1.4.7"]


def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["systemvalue1"] = ""
    ret_code = -1

    owner = runcontext.get_extra_var('MONGODB_OWNER')
    mongod = runcontext.get_extra_var('MONGODB_MONGOD')
    if 'root' != owner:
        mongod_get_version = '''su - %s -s /bin/bash -c "%s --version | grep \"db version\""''' % (owner, mongod)
    else:
        mongod_get_version = '''%s --version | grep "db version"''' % mongod

    runcontext.log_debug("mongod_get_version: %s" % mongod_get_version)
    mongodb_version = run_cmd(mongod_get_version)
    runcontext.sleep()
    pattern = re.compile(r'(?P<db>\s*db\s+version\s+)(?P<version>.+)')
    m = pattern.match(mongodb_version)
    if m is None:
        raise Exception('There is error when get mongodb version!')
    mongodb_version = m.group('version').lstrip('v').strip()
    db_version_list = mongodb_version.split('.')
    version_great_than_3_0 = dbversion_compare(db_version_list)

    if version_great_than_3_0:
        tmp_vars["checkresult1"] = "pass"
        tmp_vars["systemvalue1"] = "db version v%s" % mongodb_version
    else:
        var_mongo_parser = runcontext.get_parameter('_mongo_conf_parse')
        ret_cmd_param, ret_cmd_value = var_mongo_parser.get_cmd_value(["--set_parameter"])
        runcontext.log_debug("ret_cmd_param: %s  ret_cmd_value: %s" % (ret_cmd_param, ret_cmd_value))
        v_length = len(ret_cmd_value)
        if v_length != 0:
            if isinstance(ret_cmd_value, str):
                tmp_vars["systemvalue1"] = ' '.join([ret_cmd_param, ret_cmd_value])
                if ret_cmd_value in ["enableLocalhostAuthBypass=0", "enableLocalhostAuthBypass=false"]:
                    tmp_vars["checkresult1"] = "pass"
            else:
                for i in range(v_length):
                    if "enableLocalhostAuthBypass=" in ret_cmd_value[i]:
                        tmp_vars["systemvalue1"] = ' '.join([ret_cmd_param[i], ret_cmd_value[i]])
                        if ret_cmd_value[i] in ["enableLocalhostAuthBypass=0", "enableLocalhostAuthBypass=false"]:
                            tmp_vars["checkresult1"] = "pass"
                        break
                if tmp_vars["systemvalue1"] == "":
                    tmp_vars["systemvalue1"] = ' '.join([ret_cmd_param[-1], ret_cmd_value[-1]])
        else:
            yml_key_list = ["set_parameter", "enableLocalhostAuthBypass"]
            key_list = ["set_parameter", "enableLocalhostAuthBypass"]
            ret_conf_value = var_mongo_parser.get_conf_value(yml_key_list, key_list)
            runcontext.log_debug("ret_conf_value: %s" % str(ret_conf_value))

            if ret_conf_value is not None:
                if var_mongo_parser.type == 'str':
                    tmp_vars["systemvalue1"] = '='.join(key_list) + '=' + ret_conf_value
                else:
                    tmp_vars["systemvalue1"] = '.'.join(yml_key_list) + ': ' + str(ret_conf_value).lower()

                if str(ret_conf_value).lower() in ['false', 'no', 'off', '0']:
                    tmp_vars["checkresult1"] = "pass"

    if tmp_vars["checkresult1"] == "pass":
        ret_code = 0
        tmp_vars["policyresult"] = "pass"

    rc, out = curtemplate.render(runcontext.get_template_file('r_2_1_4_7.j2'), **tmp_vars)

    if rc == False:
        return (-2, "Failed to render template.%s" % out)

    return ret_code, out
